@@ -8,7 +8,7 @@
  ****************************************************************************/
 
 /*
- * <$Id: conv_stf.c,v 3.19 2002/03/04 12:34:10 dillema Exp $>
+ * <$Id: conv_stf.c,v 3.20 2002/12/11 16:39:49 dillema Exp $>
  */
 
 #include "totd.h"
@@ -40,9 +40,9 @@ RRset *conv_stf_owner_rrset (RRset *rrs) {
 	own = rrset_owner (rrs);
 
 	/* XXX need a strdname() routine for this */
-	ostop = strstr(own, "\007IN-ADDR\004ARPA");
+	ostop = (u_char *) strstr((char *)own, "\007IN-ADDR\004ARPA");
 	if (!ostop)
-		ostop = strstr(own, "\007in-addr\004arpa");
+		ostop = (u_char *)strstr((char *) own, "\007in-addr\004arpa");
 
 	if (!ostop) {
 		/*
@@ -77,7 +77,7 @@ RRset *conv_stf_owner_rrset (RRset *rrs) {
 			own += *own + 1;
 		}
 
-		sprintf(op, "%s", STF_SUFFIX);
+		sprintf((char *)op, "%s", STF_SUFFIX);
 		/* point to end (after null char) */
 		op += strlen(STF_SUFFIX) + 1;
 		owner_len = op - owner;
@@ -136,11 +136,11 @@ int conv_stf_ptr (u_char *qname) {
 	syslog (LOG_DEBUG, "%s: start", fn);
 
 	/* to small to convert? */
-	if (strlen(qname) < 8)
+	if (strlen((char *)qname) < 8)
 		return -1;
 
 	qn4 = qname4;
-	qname_len = strlen(qname) + 1;
+	qname_len = strlen((char *)qname) + 1;
 	qn6 = qname + qname_len - off*4 - 8;
 	if (qn6 < qname)
 		qn6 = qname + 1;
@@ -169,7 +169,7 @@ int conv_stf_ptr (u_char *qname) {
 		qn4 = qn4 + *qn4 + 1;
 	}
 	sprintf((char *)qn4, "%s", "\007in-addr\004arpa");
-	strlcpy(qname, qname4, MAX_DNAME);
+	strlcpy((char *)qname, (char *)qname4, MAX_DNAME);
 
 	if (T.debug) {
 		dname_decompress (str, MAX_DNAME, qname, NULL, NULL, NULL);
