@@ -8,7 +8,7 @@
  ****************************************************************************/
 
 /*
- * <$Id: ne_mesg.c,v 3.45 2002/03/06 14:56:02 dillema Exp $>
+ * <$Id: ne_mesg.c,v 3.48 2002/12/10 22:31:18 dillema Exp $>
  */
 
 #include "totd.h"
@@ -464,15 +464,21 @@ int mesg_parse (u_char *msg, int msg_len, G_List *an_list, G_List *ns_list,
 
 u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 			  u_char *m_head, u_char *m_tail, int *written) {
-	int token_len, written_len;
+	int token_len, written_len, iter;
 	u_char *cp, *next;
+	int pktsiz = m_tail - m_head;
 
 	next = NULL;
 	written_len = token_len = 0;
 	for (cp = dname; *cp; cp += token_len) {
+		iter = 0;
+	  top:
 		if ((*cp & DNCMP_MASK) == DNCMP_MASK) {
 			uint16_t ui;
 
+			if (iter++ >= pktsiz) /* we're probably in a loop. */
+				return NULL;
+
 			if (!m_head || !m_tail) /* irregular redirect */
 				return NULL;
 
@@ -484,6 +490,8 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 			cp = m_head + ui;
 			if (cp < m_head || m_tail < cp)
 				return NULL;
+
+			goto top;
 		}
 
 		token_len = labellen(cp);
@@ -492,11 +500,13 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 		else
 			token_len++;
 
-		if (T.debug > 3)
+		if (T.debug > 4)
 			syslog (LOG_DEBUG, "token_len: %d", token_len);
 
 		if (written_len + token_len >= buflen)
 			return NULL; /* buffer overrun */
+		if (cp + token_len > m_tail)
+			return NULL; /* out of bounds */
 
 		if (written) {
 			/* non-printable dname string */
@@ -543,7 +553,7 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 		*written = written_len + 1;
 
 	if (!next)
-		next = cp;
+		next = cp+1;
 
 	return next;
 }
