@@ -8,7 +8,7 @@
  ****************************************************************************/
 
 /*
- * <$Id: protos.h,v 3.59 2002/03/06 14:56:02 dillema Exp $>
+ * <$Id: protos.h,v 3.60 2002/12/11 16:39:50 dillema Exp $>
  */
 
 #if !HAVE_STRLCPY
@@ -105,6 +105,7 @@ void ev_to_data_free (Ev_TO_Data *);
 void ev_to_data_free_v (void *);
 void ev_to_finish (void);
 int ev_to_handler_ifcheck (Ev_TO_Data *);
+int ev_to_register_ifcheck (void);
 int ev_udp_in_eq (void *, void *);
 void ev_udp_in_data_free (Ev_UDP_In_Data * _p);
 void ev_udp_in_data_free_v (void *_vp);
@@ -184,7 +185,7 @@ char *string_rclass (uint16_t);
 char *string_rtype (uint16_t);
 
 void totd_eventloop (void);
-void totd_exit (int);
+int totd_exit (int);
 
 int conv_scoped_query (Context *);
 int conv_scoped_conf (const char *, const char *, int);
