@@ -8,7 +8,7 @@
  ****************************************************************************/
 
 /*
- * <$Id: request.c,v 3.94 2002/03/06 14:56:02 dillema Exp $>
+ * <$Id: request.c,v 3.95 2002/12/09 18:41:43 dillema Exp $>
  */
 
 #include "totd.h"
@@ -283,13 +283,12 @@ int recursive_process (Context *cont) {
 	 */
 
 	if (cont->child) {
-		/*
-		 * we need the resource type of the response message
-		 */
 		u_char *cp;
 
 		if (cont->child->mesg.hdr->rcode != RC_OK &&
-	    	    cont->child->mesg.hdr->rcode != RC_NXDOMAIN) {
+	    	    cont->child->mesg.hdr->rcode != RC_NXDOMAIN &&
+		    cont->child->mesg.hdr->rcode != RC_SERVERERR) {
+			/* RC_SERVERERR is handled below to allow hack */
 			cont->mesg.hdr->rcode = cont->child->mesg.hdr->rcode;
 			return 1;
 		}
@@ -297,8 +296,10 @@ int recursive_process (Context *cont) {
 		cp = mesg_skip_dname (cont->child->mesg.p + sizeof (Mesg_Hdr),
 			cont->child->mesg.p + cont->child->mesg_len);
 		if (cp) {
+			/*
+		 	 * we need the resource type of the response message
+		 	 */
 			GETSHORT (rtype, cp);
-			/* got what we wanted in rtype, so go on... */
 		} else {
 			syslog(LOG_INFO, "%s: malformed message", fn);
 			return -1;
@@ -315,6 +316,10 @@ int recursive_process (Context *cont) {
 
 	syslog (LOG_DEBUG, "%s: work state: %d", fn, cont->work_state);
 
+	/*
+	 * In case of a truncated response, we'll try the same request 
+	 * again using TCP.
+	 */
 	switch_to_tcp = 0;
 	if (cont->child->mesg.hdr->tc == 1) {
 		syslog (LOG_DEBUG, "%s: Truncated Message", fn);
@@ -330,6 +335,23 @@ int recursive_process (Context *cont) {
 		}
 	}
 
+	/*
+	 * Some misbehaving old nameservers simply fail to process a 
+	 * AAAA query and return a RC_SERVERERR (aka. SERVFAIL)
+	 * error response. If such is the case, we hack around it
+	 * and let totd pretend it got a proper NXDOMAIN response.
+	 */
+	if (cont->child->mesg.hdr->rcode == RC_SERVERERR) { 
+	    	if (cont->work_state == FORWARD_REQUEST &&
+		    (qtype == RT_AAAA || qtype == RT_A6)) {
+			cont->child->mesg.hdr->rcode = RC_NXDOMAIN;
+		} else {
+			/* behave as normal in case of failure (see above) */
+                        cont->mesg.hdr->rcode = cont->child->mesg.hdr->rcode;
+                        return 1;
+		}
+	}
+
 	/*
 	 * we changed a AAAA or A6 query into a A query, now construct
 	 * AAAA or A6 replies with prefix to match original request.
@@ -373,8 +395,9 @@ int recursive_process (Context *cont) {
 	 * some answers...
 	 */
 	if (cont->work_state == FORWARD_REQUEST) {
-		if (T.prefixnum && (!cont->child->mesg.hdr ||
-		    (cont->child->mesg.hdr->ancnt == htons (0)))) {
+		if (T.prefixnum && (cont->child->mesg.hdr->rcode == RC_NXDOMAIN ||
+		    (cont->child->mesg.hdr->rcode == RC_OK && (!cont->child->mesg.hdr ||
+                         cont->child->mesg.hdr->ancnt == htons(0))))) {
 			/* We got no answers in the reply we got to our query. 
 		 	 * We check here whether we need to modify the query
 		 	 * to get answers we can transform into (pseudo-)
