@@ -8,13 +8,13 @@
  ****************************************************************************/
 
 /*
- * <$Id: totd.c,v 3.63 2002/03/06 14:56:03 dillema Exp $>
+ * <$Id: totd.c,v 3.66 2003/01/27 16:22:46 dillema Exp $>
  */
 
 #include "totd.h"
 
 struct ToT T;
-char *version = "Trick or Treat Daemon (totd) version 1.3";
+char *version = "Trick or Treat Daemon (totd) version 1.4-beta";
 
 void usage () {
 	printf ("%s\n\
@@ -26,6 +26,9 @@ Usage: totd [-6|-no6|-4|-no4|-64|-dn|-v|-q|-p <prefix>|-c <filename>]\n\
 -dn         : debug mode (no fork / loglevel = n)\n\
 -v          : verbose\n\
 -q          : quiet\n\
+-u <user>   : username or uid totd should run at, after startup\n\
+-g <group>  : groupname or gid totd should run at, after startup\n\
+-t <dir>    : put totd in <dir> chroot() cage\n\
 -p <prefix> : a prefix to use for totd tricks; more than one allowed\n\
 -c <file>   : specify alternative totd configfile, default=%s\n\
 \n\
@@ -42,7 +45,7 @@ totd %s use IPv4 because it was compiled %s USE_INET4 option in config.h\n",
 		(V6 (1) + 0) ? "with" : "without"
 	);
 
-	exit (1);
+	totd_exit (EXIT_FAILURE);
 }
 
 const char *hex = "0123456789abcdef";
@@ -52,17 +55,12 @@ int main (int argc, char **argv) {
 	int i;
 
 	/* initialize global totd structure */
-	T.tot_uid = getuid();
-	T.tot_gid = getgid();
-	T.su = 0;
-	T.totuser = NULL;
-	if (!T.tot_uid) {
-                /* got started as root user */
-		T.su = 1;
-#ifdef TOT_USER
-		T.totuser = TOT_USER;
-#endif
-	}
+	T.uid = getuid();
+	T.gid = getgid();
+
+	T.user = NULL;
+	T.group = NULL;
+	T.rootdir = NULL;
 
 	/* fill in default config values */
 	T.ip4 = V4 (1) + 0;	/* bit ugly, but short */
@@ -71,6 +69,7 @@ int main (int argc, char **argv) {
 	T.quiet = 0;
 	T.debug = 0;
 	T.prefixnum = 0;
+	T.rescan_iflist = 1;
 	T.stf = 0;
 	T.retry_interval = 300;
 	T.port = PORT_SRV;
@@ -82,7 +81,7 @@ int main (int argc, char **argv) {
 
 	/* make sure these start out empty */
 	for (i = 0; i < MAXPREFIXES; i++) {
-		T.tot_prefix[i][0] = '\0';
+		T.prefix[i][0] = '\0';
 	}
 
 	/* list of forwarders */
@@ -112,14 +111,20 @@ int main (int argc, char **argv) {
 			T.ip4 = 0;
 		else if (!strcmp (argv[i], "-no6"))
 			T.ip6 = 0;
-		else if (!strcmp (argv[i], "-c")) {
+		else if (!strcmp (argv[i], "-c"))
 			T.configfile = strdup(argv[++i]);
-		} else if (!strcmp (argv[i], "-p")) {
+		else if (!strcmp (argv[i], "-p")) {
 			if (conv_trick_conf((u_char *)argv[++i])) {
 			    syslog (LOG_ERR, "invalid prefix on command line: %s", argv[++i]);
 			    usage(1);
 			}
-		} else if (!strcmp (argv[i], "-h"))
+		} else if (!strcmp (argv[i], "-u"))
+			T.user = strdup(argv[++i]);
+		else if (!strcmp (argv[i], "-g"))
+			T.group = strdup(argv[++i]);
+		else if (!strcmp (argv[i], "-t"))
+			T.rootdir = strdup(argv[++i]);
+		else if (!strcmp (argv[i], "-h"))
 			usage (1);
 		else if (!strcmp (argv[i], "-v"))
 			T.quiet = -1;
@@ -146,16 +151,77 @@ int main (int argc, char **argv) {
 		if (T.quiet > 0)
 			setlogmask (LOG_UPTO (LOG_ERR));
 		else if (!T.quiet)
-        		setlogmask (LOG_UPTO (LOG_INFO));
+		setlogmask (LOG_UPTO (LOG_INFO));
 		else
-        		setlogmask (LOG_UPTO (LOG_DEBUG));
+			setlogmask (LOG_UPTO (LOG_DEBUG));
 	} else {
 		if (T.quiet > 0)
 			setlogmask (LOG_UPTO (LOG_ERR));
 		else if (!T.quiet)
 			setlogmask (LOG_UPTO (LOG_WARNING));
 		else if (T.quiet < 0)
-        		setlogmask (LOG_UPTO (LOG_INFO));
+	       		setlogmask (LOG_UPTO (LOG_INFO));
+	}
+
+	if (T.user) {
+                if (isdigit(T.user[0])) {
+			T.uid = atoi(T.user);
+			pwd_p = NULL;
+		} else {
+			pwd_p = getpwnam (T.user);
+		}
+	} else
+		pwd_p = getpwuid (T.uid);
+
+	if (pwd_p) {
+		syslog (LOG_INFO, "Found user record of %s; uid: %d gid: %d",
+			pwd_p->pw_name, pwd_p->pw_uid, pwd_p->pw_gid);
+		if (T.uid && T.uid != pwd_p->pw_uid) {
+			syslog (LOG_ERR, "Need root privileges to change user \
+to: %s", T.user);
+			totd_exit (EXIT_FAILURE);
+		} else {
+			T.uid = pwd_p->pw_uid;
+		}
+	} else {
+		syslog (LOG_INFO, "can't find user record of %s", T.user);
+	}
+
+	if (T.group) {
+		if (isdigit(T.group[0])) {
+			T.gid = atoi(T.group);
+		} else {
+        		struct group *grp_p;
+
+			grp_p = getgrnam(T.group);
+			if (!grp_p) {
+				syslog(LOG_ERR, "group `%s' unknown", T.group);
+				totd_exit (EXIT_FAILURE);
+			}
+			T.gid = grp_p->gr_gid;
+		}
+	} else {
+		if (pwd_p)
+			T.gid = pwd_p->pw_gid;
+		else
+			T.gid = getgid();
+	}
+
+	/* close any open descriptors */
+	endpwent();
+	endgrent();
+
+	/* chroot() cage totd if asked for */
+	if (T.rootdir) {
+                if (chroot(T.rootdir) < 0) {
+                        syslog (LOG_ERR, "chroot %s failed: %m", T.rootdir);
+                        totd_exit(EXIT_FAILURE);
+                } 
+                syslog (LOG_INFO, "chrooted to %s", T.rootdir);
+                if (chdir("/") < 0) {
+                        syslog (LOG_ERR, "chdir(\"/\") failed: %m");
+                        totd_exit(EXIT_FAILURE); 
+                }
 	}
 
 	/*
@@ -165,42 +231,16 @@ int main (int argc, char **argv) {
 	if (read_config (T.configfile)) {
 	    syslog (LOG_ERR, "Configuration failure");
 	    syslog (LOG_INFO, "Check log, or try -d option for debug mode...");
-	    exit(1);
+	    totd_exit (EXIT_FAILURE);
 	}
 
-	/*
-	 * Let's do some (in)sanity config checking now
-	 */
-	if (T.port < IPPORT_RESERVED && T.tot_uid) {
-		syslog (LOG_ERR, "Need root privileges to run at reserved port %d", T.port);
-		syslog (LOG_INFO, "Start totd as root or configure it to use \
-a non-reserved port");
-		exit(1);
-	}
-
-	if (T.totuser)
-		pwd_p = getpwnam (T.totuser);
-	else
-		pwd_p = getpwuid (T.tot_uid);
-
-	if (pwd_p) {
-		syslog (LOG_INFO, "Found user record of %s; uid: %d gid: %d",
-			pwd_p->pw_name, pwd_p->pw_uid, pwd_p->pw_gid);
-		if (T.tot_uid && T.tot_uid != pwd_p->pw_uid) {
-			syslog (LOG_ERR, "Need root privileges to change user to: %s", T.totuser);
-			syslog (LOG_INFO, "Check also `totuser' configfile option");
-			exit(1);
-		} else {
-			T.totuser = strdup(pwd_p->pw_name);
-			T.tot_uid = pwd_p->pw_uid;
-			T.tot_gid = pwd_p->pw_gid;
-			/* T.su = 1; */
-		}
-	} else {
-		syslog (LOG_ERR, "can't find user record of %s", T.totuser);
-		exit(1);
+	/* we cannot rebind to reserved ports if we are not root */
+	if (T.rescan_iflist == 1 && T.uid && T.port < IPPORT_RESERVED) {
+		syslog(LOG_WARNING, "Disabling rescanning of network interfaces");
+		T.rescan_iflist = 0;
 	}
 
+	/* Check for pidfile conflicts */
 	if (T.pidfile) {
 		FILE *pid_fp;
 		pid_t pid;
@@ -219,7 +259,8 @@ a non-reserved port");
 				syslog (LOG_INFO, "There can be another totd \
 running. Please make sure there's no totd already running. Then delete the \
 file %s and try again.", T.pidfile);
-				exit (1);
+				fclose (pid_fp);
+				totd_exit (EXIT_FAILURE);
 			}
 			fclose (pid_fp);
 		}
@@ -237,7 +278,7 @@ file %s and try again.", T.pidfile);
 this OS, due to IPv4 mapped addresses");
 		syslog(LOG_INFO, "Will always accept IPv4 *and* IPv6 requests.\
 Bailing out, so that you can explicitly tell me to do so");
-		exit(1);
+		totd_exit (EXIT_FAILURE);
 #endif
 	}
 
@@ -248,7 +289,7 @@ Bailing out, so that you can explicitly tell me to do so");
 wildcard binding when IPv6 is enabled.");
 			syslog(LOG_INFO, "Please remove `interfaces' \
 specification in your config file.");
-			exit(1);
+			totd_exit (EXIT_FAILURE);
 		}
 #endif
 #ifdef NEEDSV4MAPPED
@@ -263,7 +304,7 @@ will not bind to wildcard IPv4 socket.");
 		syslog (LOG_INFO, "IPv4 activated");
 #else
 		syslog (LOG_ERR, "IPv4 support is not compiled in");
-		exit (1);
+		totd_exit (EXIT_FAILURE);
 #endif
 	}
 
@@ -272,14 +313,14 @@ will not bind to wildcard IPv4 socket.");
 		syslog (LOG_INFO, "IPv6 activated");
 #else
 		syslog (LOG_ERR, "IPv6 support is not compiled in");
-		exit (1);
+		totd_exit (EXIT_FAILURE);
 #endif
 	}
 
 	if (!T.ip4 && !T.ip6) {
 		syslog (LOG_ERR, "all supported protocols are deactivated; \
 what do you want me to do then?");
-		exit (1);
+		totd_exit (EXIT_FAILURE);
 	}
 
 #ifdef SCOPED_REWRITE
@@ -287,7 +328,7 @@ what do you want me to do then?");
 		syslog (LOG_ERR, "Scoped address rewriting currently not \
 implemented when wildcard sockets are used. Please use `interfaces' keyword \
 in your config file or remove `scoped' keyword");
-		exit (1);
+		totd_exit (EXIT_FAILURE);
 	}
 #endif
 
@@ -298,45 +339,87 @@ in your config file or remove `scoped' keyword");
 	    return -1;
 	}
 
-	if (!T.debug) {
-		/* purge tty */
-		if (fork())
-			exit (0); /* let parent exit */
-		/* child goes into background */
-		close (0);
-		close (1);
-		close (2);
-		open ("/dev/null", O_RDONLY);
-		dup2 (0, 1);
-		dup2 (0, 2);
-		i = open ("/dev/tty", O_RDWR);
-		if (i >= 0) {
-			ioctl (i, TIOCNOTTY, (char *) 0);
-			close (i);
+	/* initialize each event routine */
+	ev_dup_init ();
+	if (ev_signal_init () < 0) {
+		syslog (LOG_ERR, "Signal event handling  initialize failed");
+		totd_exit (EXIT_FAILURE);
+	}
+
+	if (ev_to_init () < 0) {
+		syslog (LOG_ERR, "Timeout event handling initialize failed");
+		totd_exit (EXIT_FAILURE);
+	}
+
+	if (ev_tcp_conn_in_init () < 0) {
+		syslog (LOG_ERR, "TCP connection initialize failed");
+		totd_exit (EXIT_FAILURE);
+	}
+
+	if (net_init_socketlist(T.port) < 0) {
+		syslog (LOG_ERR, "Init list of sockets failed");
+		totd_exit (EXIT_FAILURE);
+	}
+
+	if (net_bind_socketlist() <= 0) {
+		if (!T.rescan_iflist) {
+			syslog (LOG_ERR, "Could not open any sockets");
+			totd_exit (EXIT_FAILURE);
+		} else {
+			syslog (LOG_WARNING, "Could not open any sockets");
+			syslog (LOG_WARNING, "Maybe later??? Continuing");
 		}
 	}
 
+	if (ev_udp_in_init () < 0) {
+		syslog (LOG_ERR, "UDP initialize failed");
+		totd_exit (EXIT_FAILURE);
+	}
+	if (ev_tcp_out_init () < 0) {
+		syslog (LOG_ERR, "TCP output routine initialize failed");
+		totd_exit (EXIT_FAILURE);
+	}
+
+	/* drop root privs */
+	if (setgid(T.gid) < 0) {
+		syslog (LOG_ERR, "setgid to %d failed", T.gid);
+		totd_exit (EXIT_FAILURE);
+	}
+	if (setuid(T.uid) < 0) {
+		syslog (LOG_ERR, "setuid to %d failed", T.uid);
+		totd_exit (EXIT_FAILURE);
+	}
+
+	if (T.rescan_iflist) {
+		if (ev_to_register_ifcheck () < 0) {
+			syslog (LOG_ERR, "Registering Interface Check Event failed");
+			totd_exit (EXIT_FAILURE);
+		}
+	}
+
+	if (!T.debug) {
+		if (daemon(0,0))
+			totd_exit (EXIT_FAILURE);
+		else
+			syslog (LOG_INFO, "totd successfully daemonized");
+	}
+
 	/* as detached child, we can now write pid file */
 	if (T.pidfile) {
 		FILE *pid_fp;
 		pid_fp = fopen (T.pidfile, "w");
 		if (!pid_fp) {
-			syslog (LOG_ERR, "can't open pid file");
-			exit(1);
+			syslog (LOG_ERR, "can't open pid file \"%s\"", T.pidfile);
+			totd_exit (EXIT_FAILURE);
 		}
 		fprintf (pid_fp, "%d", getpid ());
+		syslog (LOG_INFO, "wrote pid %d to file %s", getpid (), T.pidfile);
 		fclose (pid_fp);
 	}
 
-	/*
-	 * let's give up root privileges (if we have them),
-	 * only go back to them when needed.
-	 */
-	ROOTPRIV_DISABLE ("Startup successful.");
-
 	syslog (LOG_INFO, "totd started");
 	totd_eventloop ();
-	exit (0);
+	return (totd_exit (0));
 }
 
 void totd_eventloop (void) {
@@ -348,50 +431,7 @@ void totd_eventloop (void) {
 
 #ifdef DBMALLOC
 	unsigned long histid1, histid2, orig_size, current_size;
-#endif
 
-	/* initialize each event routine */
-	ev_dup_init ();
-	if (ev_signal_init () < 0) {
-		syslog (LOG_ERR, "Signal event handling  initialize failed");
-		totd_exit (-1);
-	}
-	if (ev_to_init () < 0) {
-		syslog (LOG_ERR, "Timeout event handling initialize failed");
-		totd_exit (-1);
-	}
-	if (T.port < IPPORT_RESERVED) {
-		ROOTPRIV_ENABLE ("Event: initialize reserve sockets");
-	}
-
-	if (ev_tcp_conn_in_init () < 0) {
-		syslog (LOG_ERR, "TCP connection initialize failed");
-		totd_exit (-1);
-	}
-
-	if (net_init_socketlist(T.port) < 0) {
-		syslog (LOG_ERR, "Init list of sockets failed");
-		totd_exit (-1);
-	}
-
-	if (net_bind_socketlist() < 0) {
-		syslog (LOG_ERR, "Binding list of sockets failed");
-		totd_exit (-1);
-	}
-
-	if (ev_udp_in_init () < 0) {
-		syslog (LOG_ERR, "UDP initialize failed");
-		totd_exit (-1);
-	}
-	if (ev_tcp_out_init () < 0) {
-		syslog (LOG_ERR, "TCP output routine initialize failed");
-		totd_exit (-1);
-	}
-	if (T.port < IPPORT_RESERVED) {
-		ROOTPRIV_DISABLE ("Event: initialize reserved sockets");
-	}
-
-#ifdef DBMALLOC
 	/* Malloc debugging */
 	if (T.debug) 
 		malloc_dump (2);
@@ -400,10 +440,6 @@ void totd_eventloop (void) {
 #endif
 
 	while (1) {		/* main loop */
-		if (T.port < IPPORT_RESERVED) {
-			ROOTPRIV_DISABLE ("Event: mainloop");
-		}
-
 		if (T.debug > 2)
 			syslog (LOG_DEBUG, "main loop: start");
 
@@ -477,7 +513,7 @@ void totd_eventloop (void) {
 			} else {
 				syslog (LOG_ERR, "%s: select(): %m", fn);
 				if (net_reinit_socketlist (T.port, 1) < 0)
-					totd_exit(-1);
+					totd_exit (EXIT_FAILURE);
 				sleep (1);
 			}
 		} else {
@@ -503,9 +539,7 @@ void totd_eventloop (void) {
 	}			/* while(1) */
 }
 
-void totd_exit (int status) {
-
-	ROOTPRIV_ENABLE ("exit");
+int totd_exit (int status) {
 
 	/* finish in reverse order of initialize */
 	ev_tcp_out_finish ();
@@ -520,10 +554,11 @@ void totd_exit (int status) {
 	if (T.pidfile)
 		unlink (T.pidfile);
 
-	ROOTPRIV_DISABLE ("exit");
-	setlogmask (LOG_UPTO (LOG_INFO));
-	syslog (LOG_INFO, "terminated. %s euid = %d",
-		(T.su) ? " (in su mode)" : "", geteuid ());
+	if (status != EXIT_SUCCESS) {
+		syslog (LOG_ERR, "terminated with error");
+		fprintf (stderr, "totd terminated with error, \
+check system logs for details or run totd in debug mode.\n");
+	}
 
 	exit (status);
 }
